@@ -39,6 +39,99 @@
 #include "shared/log.h"
 #include "shared/rt.h"
 
+static const struct a2dp_bit_mapping a2dp_aac_channels[] = {
+	{ AAC_CHANNEL_MODE_MONO, 1 },
+	{ AAC_CHANNEL_MODE_STEREO, 2 },
+	{ AAC_CHANNEL_MODE_5_1, 6 },
+	{ AAC_CHANNEL_MODE_7_1, 8 },
+	{ 0 }
+};
+
+static const struct a2dp_bit_mapping a2dp_aac_samplings[] = {
+	{ AAC_SAMPLING_FREQ_8000, 8000 },
+	{ AAC_SAMPLING_FREQ_11025, 11025 },
+	{ AAC_SAMPLING_FREQ_12000, 12000 },
+	{ AAC_SAMPLING_FREQ_16000, 16000 },
+	{ AAC_SAMPLING_FREQ_22050, 22050 },
+	{ AAC_SAMPLING_FREQ_24000, 24000 },
+	{ AAC_SAMPLING_FREQ_32000, 32000 },
+	{ AAC_SAMPLING_FREQ_44100, 44100 },
+	{ AAC_SAMPLING_FREQ_48000, 48000 },
+	{ AAC_SAMPLING_FREQ_64000, 64000 },
+	{ AAC_SAMPLING_FREQ_88200, 88200 },
+	{ AAC_SAMPLING_FREQ_96000, 96000 },
+	{ 0 }
+};
+
+static void a2dp_aac_caps_intersect(
+		void *capabilities,
+		const void *mask) {
+
+	const a2dp_aac_t *caps_mask = mask;
+	a2dp_aac_t *caps = capabilities;
+
+	int rate = MIN(A2DP_AAC_GET_BITRATE(*caps), A2DP_AAC_GET_BITRATE(*caps_mask));
+
+	a2dp_caps_bitwise_intersect(caps, caps_mask, sizeof(*caps));
+	A2DP_AAC_SET_BITRATE(*caps, rate);
+
+}
+
+static int a2dp_aac_caps_foreach_channel_mode(
+		const void *capabilities,
+		enum a2dp_stream stream,
+		a2dp_bit_mapping_foreach_func func,
+		void *userdata) {
+	const a2dp_aac_t *caps = capabilities;
+	if (stream == A2DP_MAIN)
+		return a2dp_bit_mapping_foreach(a2dp_aac_channels, caps->channel_mode, func, userdata);
+	return -1;
+}
+
+static int a2dp_aac_caps_foreach_sampling_freq(
+		const void *capabilities,
+		enum a2dp_stream stream,
+		a2dp_bit_mapping_foreach_func func,
+		void *userdata) {
+	const a2dp_aac_t *caps = capabilities;
+	if (stream == A2DP_MAIN) {
+		const uint16_t sampling_freq = A2DP_AAC_GET_SAMPLING_FREQ(*caps);
+		return a2dp_bit_mapping_foreach(a2dp_aac_samplings, sampling_freq, func, userdata);
+	}
+	return -1;
+}
+
+static void a2dp_aac_caps_select_channel_mode(
+		void *capabilities,
+		enum a2dp_stream stream,
+		unsigned int channels) {
+	a2dp_aac_t *caps = capabilities;
+	if (stream == A2DP_MAIN)
+		caps->channel_mode = a2dp_bit_mapping_lookup_value(a2dp_aac_channels,
+				caps->channel_mode, channels);
+}
+
+static void a2dp_aac_caps_select_sampling_freq(
+		void *capabilities,
+		enum a2dp_stream stream,
+		unsigned int frequency) {
+	a2dp_aac_t *caps = capabilities;
+	if (stream == A2DP_MAIN) {
+		const uint16_t sampling_freq = a2dp_bit_mapping_lookup_value(a2dp_aac_samplings,
+				A2DP_AAC_GET_SAMPLING_FREQ(*caps), frequency);
+		A2DP_AAC_SET_SAMPLING_FREQ(*caps, sampling_freq);
+	}
+}
+
+static struct a2dp_caps_helpers a2dp_aac_caps_helpers = {
+	.intersect = a2dp_aac_caps_intersect,
+	.has_stream = a2dp_caps_has_main_stream_only,
+	.foreach_channel_mode = a2dp_aac_caps_foreach_channel_mode,
+	.foreach_sampling_freq = a2dp_aac_caps_foreach_sampling_freq,
+	.select_channel_mode = a2dp_aac_caps_select_channel_mode,
+	.select_sampling_freq = a2dp_aac_caps_select_sampling_freq,
+};
+
 static unsigned int a2dp_aac_get_fdk_vbr_mode(
 		unsigned int channels, unsigned int bitrate) {
 	static const unsigned int modes[][5] = {
@@ -72,7 +165,7 @@ void *a2dp_aac_enc_thread(struct ba_transport_pcm *t_pcm) {
 	const unsigned int samplerate = t_pcm->sampling;
 
 	/* create AAC encoder without the Meta Data module */
-	if ((err = aacEncOpen(&handle, 0x07, channels)) != AACENC_OK) {
+	if ((err = aacEncOpen(&handle, 0x0F, channels)) != AACENC_OK) {
 		error("Couldn't open AAC encoder: %s", aacenc_strerror(err));
 		goto fail_open;
 	}
@@ -108,17 +201,17 @@ void *a2dp_aac_enc_thread(struct ba_transport_pcm *t_pcm) {
 	}
 
 	unsigned int channel_mode = MODE_1;
-	switch (configuration->channels) {
-	case AAC_CHANNELS_1:
+	switch (configuration->channel_mode) {
+	case AAC_CHANNEL_MODE_MONO:
 		channel_mode = MODE_1;
 		break;
-	case AAC_CHANNELS_2:
+	case AAC_CHANNEL_MODE_STEREO:
 		channel_mode = MODE_2;
 		break;
-	case AAC_CHANNELS_6:
+	case AAC_CHANNEL_MODE_5_1:
 		channel_mode = MODE_1_2_2_1;
 		break;
-	case AAC_CHANNELS_8:
+	case AAC_CHANNEL_MODE_7_1:
 		channel_mode = MODE_1_2_2_2_1;
 		break;
 	}
@@ -247,7 +340,7 @@ void *a2dp_aac_enc_thread(struct ba_transport_pcm *t_pcm) {
 			continue;
 		}
 
-		while ((in_args.numInSamples = ffb_len_out(&pcm)) > 0) {
+		while ((in_args.numInSamples = ffb_len_out(&pcm)) > (int)aacinf.inputChannels) {
 
 			if ((err = aacEncEncode(handle, &in_buf, &out_buf, &in_args, &out_args)) != AACENC_OK)
 				error("AAC encoding error: %s", aacenc_strerror(err));
@@ -293,7 +386,7 @@ void *a2dp_aac_enc_thread(struct ba_transport_pcm *t_pcm) {
 
 			}
 
-			unsigned int pcm_frames = out_args.numInSamples / channels;
+			unsigned int pcm_frames = out_args.numInSamples / aacinf.inputChannels;
 			/* keep data transfer at a constant bit rate */
 			asrsync_sync(&io.asrs, pcm_frames);
 			/* move forward RTP timestamp clock */
@@ -478,62 +571,29 @@ fail_open:
 	return NULL;
 }
 
-static const struct a2dp_channels a2dp_aac_channels[] = {
-	{ 1, AAC_CHANNELS_1 },
-	{ 2, AAC_CHANNELS_2 },
-	{ 6, AAC_CHANNELS_6 },
-	{ 8, AAC_CHANNELS_8 },
-	{ 0 },
-};
-
-static const struct a2dp_sampling a2dp_aac_samplings[] = {
-	{ 8000, AAC_SAMPLING_FREQ_8000 },
-	{ 11025, AAC_SAMPLING_FREQ_11025 },
-	{ 12000, AAC_SAMPLING_FREQ_12000 },
-	{ 16000, AAC_SAMPLING_FREQ_16000 },
-	{ 22050, AAC_SAMPLING_FREQ_22050 },
-	{ 24000, AAC_SAMPLING_FREQ_24000 },
-	{ 32000, AAC_SAMPLING_FREQ_32000 },
-	{ 44100, AAC_SAMPLING_FREQ_44100 },
-	{ 48000, AAC_SAMPLING_FREQ_48000 },
-	{ 64000, AAC_SAMPLING_FREQ_64000 },
-	{ 88200, AAC_SAMPLING_FREQ_88200 },
-	{ 96000, AAC_SAMPLING_FREQ_96000 },
-	{ 0 },
-};
-
-static int a2dp_aac_capabilities_filter(
-		const struct a2dp_codec *codec,
-		const void *capabilities_mask,
-		void *capabilities) {
-
-	(void)codec;
-	const a2dp_aac_t *caps_mask = capabilities_mask;
-	a2dp_aac_t *caps = capabilities;
-
-	int rate = MIN(A2DP_AAC_GET_BITRATE(*caps), A2DP_AAC_GET_BITRATE(*caps_mask));
-
-	for (size_t i = 0; i < sizeof(*caps); i++)
-		((uint8_t *)caps)[i] = ((uint8_t *)caps)[i] & ((uint8_t *)caps_mask)[i];
-
-	A2DP_AAC_SET_BITRATE(*caps, rate);
-
-	return 0;
-}
-
 static int a2dp_aac_configuration_select(
-		const struct a2dp_codec *codec,
+		const struct a2dp_sep *sep,
 		void *capabilities) {
 
 	a2dp_aac_t *caps = capabilities;
 	const a2dp_aac_t saved = *caps;
 
-	/* narrow capabilities to values supported by BlueALSA */
-	if (a2dp_filter_capabilities(codec, &codec->capabilities,
-				caps, sizeof(*caps)) != 0)
-		return -1;
+	/* Narrow capabilities to values supported by BlueALSA. */
+	a2dp_aac_caps_intersect(caps, &sep->config.capabilities);
 
-	if (caps->object_type & AAC_OBJECT_TYPE_MPEG4_HE2)
+	unsigned int channel_mode = 0;
+	if (a2dp_aac_caps_foreach_channel_mode(caps, A2DP_MAIN,
+				a2dp_bit_mapping_foreach_get_best_channel_mode, &channel_mode) != -1)
+		caps->channel_mode = channel_mode;
+	else {
+		error("AAC: No supported channel modes: %#x", saved.channel_mode);
+		return errno = ENOTSUP, -1;
+	}
+
+	if (caps->object_type & AAC_OBJECT_TYPE_MPEG4_HE2 &&
+			/* The HEv2 uses SBR with Parametric Stereo algorithm
+			 * which works only with stereo channel mode. */
+			channel_mode == AAC_CHANNEL_MODE_STEREO)
 		caps->object_type = AAC_OBJECT_TYPE_MPEG4_HE2;
 	else if (caps->object_type & AAC_OBJECT_TYPE_MPEG4_HE)
 		caps->object_type = AAC_OBJECT_TYPE_MPEG4_HE;
@@ -552,24 +612,16 @@ static int a2dp_aac_configuration_select(
 		return errno = ENOTSUP, -1;
 	}
 
-	const struct a2dp_sampling *sampling;
-	const uint16_t caps_frequency = A2DP_AAC_GET_FREQUENCY(*caps);
-	if ((sampling = a2dp_sampling_select(a2dp_aac_samplings, caps_frequency)) != NULL)
-		A2DP_AAC_SET_FREQUENCY(*caps, sampling->value);
+	unsigned int sampling_freq = 0;
+	if (a2dp_aac_caps_foreach_sampling_freq(caps, A2DP_MAIN,
+				a2dp_bit_mapping_foreach_get_best_sampling_freq, &sampling_freq) != -1)
+		A2DP_AAC_SET_SAMPLING_FREQ(*caps, sampling_freq);
 	else {
-		error("AAC: No supported sampling frequencies: %#x", A2DP_AAC_GET_FREQUENCY(saved));
+		error("AAC: No supported sampling frequencies: %#x", A2DP_AAC_GET_SAMPLING_FREQ(saved));
 		return errno = ENOTSUP, -1;
 	}
 
-	const struct a2dp_channels *channels;
-	if ((channels = a2dp_channels_select(a2dp_aac_channels, caps->channels)) != NULL)
-		caps->channels = channels->value;
-	else {
-		error("AAC: No supported channels: %#x", saved.channels);
-		return errno = ENOTSUP, -1;
-	}
-
-	unsigned int ba_bitrate = A2DP_AAC_GET_BITRATE(codec->capabilities.aac);
+	unsigned int ba_bitrate = A2DP_AAC_GET_BITRATE(sep->config.capabilities.aac);
 	unsigned int cap_bitrate = A2DP_AAC_GET_BITRATE(*caps);
 	if (cap_bitrate == 0)
 		/* fix bitrate value if it was not set */
@@ -583,16 +635,14 @@ static int a2dp_aac_configuration_select(
 }
 
 static int a2dp_aac_configuration_check(
-		const struct a2dp_codec *codec,
+		const struct a2dp_sep *sep,
 		const void *configuration) {
 
 	const a2dp_aac_t *conf = configuration;
 	a2dp_aac_t conf_v = *conf;
 
-	/* validate configuration against BlueALSA capabilities */
-	if (a2dp_filter_capabilities(codec, &codec->capabilities,
-				&conf_v, sizeof(conf_v)) != 0)
-		return A2DP_CHECK_ERR_SIZE;
+	/* Validate configuration against BlueALSA capabilities. */
+	a2dp_aac_caps_intersect(&conf_v, &sep->config.capabilities);
 
 	switch (conf_v.object_type) {
 	case AAC_OBJECT_TYPE_MPEG2_LC:
@@ -608,14 +658,14 @@ static int a2dp_aac_configuration_check(
 		return A2DP_CHECK_ERR_OBJECT_TYPE;
 	}
 
-	const uint16_t conf_frequency = A2DP_AAC_GET_FREQUENCY(conf_v);
-	if (a2dp_sampling_lookup(a2dp_aac_samplings, conf_frequency) == NULL) {
-		debug("AAC: Invalid sampling frequency: %#x", A2DP_AAC_GET_FREQUENCY(*conf));
+	const uint16_t conf_sampling_freq = A2DP_AAC_GET_SAMPLING_FREQ(conf_v);
+	if (a2dp_bit_mapping_lookup(a2dp_aac_samplings, conf_sampling_freq) == 0) {
+		debug("AAC: Invalid sampling frequency: %#x", A2DP_AAC_GET_SAMPLING_FREQ(*conf));
 		return A2DP_CHECK_ERR_SAMPLING;
 	}
 
-	if (a2dp_channels_lookup(a2dp_aac_channels, conf_v.channels) == NULL) {
-		debug("AAC: Invalid channel mode: %#x", conf->channels);
+	if (a2dp_bit_mapping_lookup(a2dp_aac_channels, conf_v.channel_mode) == 0) {
+		debug("AAC: Invalid channel mode: %#x", conf->channel_mode);
 		return A2DP_CHECK_ERR_CHANNEL_MODE;
 	}
 
@@ -624,24 +674,24 @@ static int a2dp_aac_configuration_check(
 
 static int a2dp_aac_transport_init(struct ba_transport *t) {
 
-	const struct a2dp_channels *channels;
-	if ((channels = a2dp_channels_lookup(a2dp_aac_channels,
-					t->a2dp.configuration.aac.channels)) == NULL)
+	unsigned int channels;
+	if ((channels = a2dp_bit_mapping_lookup(a2dp_aac_channels,
+					t->a2dp.configuration.aac.channel_mode)) == 0)
 		return -1;
 
-	const struct a2dp_sampling *sampling;
-	if ((sampling = a2dp_sampling_lookup(a2dp_aac_samplings,
-					A2DP_AAC_GET_FREQUENCY(t->a2dp.configuration.aac))) == NULL)
+	unsigned int sampling;
+	if ((sampling = a2dp_bit_mapping_lookup(a2dp_aac_samplings,
+					A2DP_AAC_GET_SAMPLING_FREQ(t->a2dp.configuration.aac))) == 0)
 		return -1;
 
 	t->a2dp.pcm.format = BA_TRANSPORT_PCM_FORMAT_S16_2LE;
-	t->a2dp.pcm.channels = channels->count;
-	t->a2dp.pcm.sampling = sampling->frequency;
+	t->a2dp.pcm.channels = channels;
+	t->a2dp.pcm.sampling = sampling;
 
 	return 0;
 }
 
-static int a2dp_aac_source_init(struct a2dp_codec *codec) {
+static int a2dp_aac_source_init(struct a2dp_sep *sep) {
 
 	LIB_INFO info[16] = {
 		[15] = { .module_id = ~FDK_NONE } };
@@ -658,25 +708,25 @@ static int a2dp_aac_source_init(struct a2dp_codec *codec) {
 	}
 
 	if (caps_aac & CAPF_ER_AAC_SCAL)
-		codec->capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_SCA;
+		sep->config.capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_SCA;
 	if (caps_sbr & CAPF_SBR_HQ)
-		codec->capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_HE;
+		sep->config.capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_HE;
 	if (caps_sbr & CAPF_SBR_PS_MPEG)
-		codec->capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_HE2;
+		sep->config.capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_HE2;
 	if (caps_aac & CAPF_ER_AAC_ELDV2)
-		codec->capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_ELD2;
+		sep->config.capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_ELD2;
 	if (caps_aac & CAPF_AAC_UNIDRC)
-		codec->capabilities.aac.drc = 1;
+		sep->config.capabilities.aac.drc = 1;
 
 	if (config.a2dp.force_mono)
-		codec->capabilities.aac.channels = AAC_CHANNELS_1;
+		sep->config.capabilities.aac.channel_mode = AAC_CHANNEL_MODE_MONO;
 	if (config.a2dp.force_44100)
-		A2DP_AAC_SET_FREQUENCY(codec->capabilities.aac, AAC_SAMPLING_FREQ_44100);
+		A2DP_AAC_SET_SAMPLING_FREQ(sep->config.capabilities.aac, AAC_SAMPLING_FREQ_44100);
 
 	if (!config.aac_prefer_vbr)
-		codec->capabilities.aac.vbr = 0;
+		sep->config.capabilities.aac.vbr = 0;
 
-	A2DP_AAC_SET_BITRATE(codec->capabilities.aac, config.aac_bitrate);
+	A2DP_AAC_SET_BITRATE(sep->config.capabilities.aac, config.aac_bitrate);
 
 	return 0;
 }
@@ -685,48 +735,50 @@ static int a2dp_aac_source_transport_start(struct ba_transport *t) {
 	return ba_transport_pcm_start(&t->a2dp.pcm, a2dp_aac_enc_thread, "ba-a2dp-aac");
 }
 
-struct a2dp_codec a2dp_aac_source = {
-	.dir = A2DP_SOURCE,
-	.codec_id = A2DP_CODEC_MPEG24,
-	.synopsis = "A2DP Source (AAC)",
-	.capabilities.aac = {
-		/* NOTE: AAC Long Term Prediction and AAC Scalable might be
-		 *       not supported by the FDK-AAC library. */
-		.object_type =
-			AAC_OBJECT_TYPE_MPEG2_LC |
-			AAC_OBJECT_TYPE_MPEG4_LC,
-		A2DP_AAC_INIT_FREQUENCY(
-				AAC_SAMPLING_FREQ_8000 |
-				AAC_SAMPLING_FREQ_11025 |
-				AAC_SAMPLING_FREQ_12000 |
-				AAC_SAMPLING_FREQ_16000 |
-				AAC_SAMPLING_FREQ_22050 |
-				AAC_SAMPLING_FREQ_24000 |
-				AAC_SAMPLING_FREQ_32000 |
-				AAC_SAMPLING_FREQ_44100 |
-				AAC_SAMPLING_FREQ_48000 |
-				AAC_SAMPLING_FREQ_64000 |
-				AAC_SAMPLING_FREQ_88200 |
-				AAC_SAMPLING_FREQ_96000)
-		.channels =
-			AAC_CHANNELS_1 |
-			AAC_CHANNELS_2 |
-			AAC_CHANNELS_6 |
-			AAC_CHANNELS_8,
-		.vbr = 1,
-		A2DP_AAC_INIT_BITRATE(320000)
+struct a2dp_sep a2dp_aac_source = {
+	.name = "A2DP Source (AAC)",
+	.config = {
+		.type = A2DP_SOURCE,
+		.codec_id = A2DP_CODEC_MPEG24,
+		.caps_size = sizeof(a2dp_aac_t),
+		.capabilities.aac = {
+			/* NOTE: AAC Long Term Prediction and AAC Scalable might be
+			 *       not supported by the FDK-AAC library. */
+			.object_type =
+				AAC_OBJECT_TYPE_MPEG2_LC |
+				AAC_OBJECT_TYPE_MPEG4_LC,
+			A2DP_AAC_INIT_SAMPLING_FREQ(
+					AAC_SAMPLING_FREQ_8000 |
+					AAC_SAMPLING_FREQ_11025 |
+					AAC_SAMPLING_FREQ_12000 |
+					AAC_SAMPLING_FREQ_16000 |
+					AAC_SAMPLING_FREQ_22050 |
+					AAC_SAMPLING_FREQ_24000 |
+					AAC_SAMPLING_FREQ_32000 |
+					AAC_SAMPLING_FREQ_44100 |
+					AAC_SAMPLING_FREQ_48000 |
+					AAC_SAMPLING_FREQ_64000 |
+					AAC_SAMPLING_FREQ_88200 |
+					AAC_SAMPLING_FREQ_96000)
+			.channel_mode =
+				AAC_CHANNEL_MODE_MONO |
+				AAC_CHANNEL_MODE_STEREO |
+				AAC_CHANNEL_MODE_5_1 |
+				AAC_CHANNEL_MODE_7_1,
+			.vbr = 1,
+			A2DP_AAC_INIT_BITRATE(320000)
+		},
 	},
-	.capabilities_size = sizeof(a2dp_aac_t),
 	.init = a2dp_aac_source_init,
-	.capabilities_filter = a2dp_aac_capabilities_filter,
 	.configuration_select = a2dp_aac_configuration_select,
 	.configuration_check = a2dp_aac_configuration_check,
 	.transport_init = a2dp_aac_transport_init,
 	.transport_start = a2dp_aac_source_transport_start,
+	.caps_helpers = &a2dp_aac_caps_helpers,
 	.enabled = true,
 };
 
-static int a2dp_aac_sink_init(struct a2dp_codec *codec) {
+static int a2dp_aac_sink_init(struct a2dp_sep *sep) {
 
 	LIB_INFO info[16] = {
 		[15] = { .module_id = ~FDK_NONE } };
@@ -745,21 +797,21 @@ static int a2dp_aac_sink_init(struct a2dp_codec *codec) {
 	}
 
 	if (caps_aac & CAPF_ER_AAC_SCAL)
-		codec->capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_SCA;
+		sep->config.capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_SCA;
 	if (caps_sbr & CAPF_SBR_HQ)
-		codec->capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_HE;
+		sep->config.capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_HE;
 	if (caps_sbr & CAPF_SBR_PS_MPEG)
-		codec->capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_HE2;
+		sep->config.capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_HE2;
 	if (caps_aac & CAPF_ER_AAC_ELDV2)
-		codec->capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_ELD2;
+		sep->config.capabilities.aac.object_type |= AAC_OBJECT_TYPE_MPEG4_ELD2;
 	if (caps_aac & CAPF_AAC_UNIDRC)
-		codec->capabilities.aac.drc = 1;
+		sep->config.capabilities.aac.drc = 1;
 	if (caps_dmx & CAPF_DMX_6_CH)
-		codec->capabilities.aac.channels |= AAC_CHANNELS_6;
+		sep->config.capabilities.aac.channel_mode |= AAC_CHANNEL_MODE_5_1;
 	if (caps_dmx & CAPF_DMX_8_CH)
-		codec->capabilities.aac.channels |= AAC_CHANNELS_8;
+		sep->config.capabilities.aac.channel_mode |= AAC_CHANNEL_MODE_7_1;
 
-	A2DP_AAC_SET_BITRATE(codec->capabilities.aac, config.aac_bitrate);
+	A2DP_AAC_SET_BITRATE(sep->config.capabilities.aac, config.aac_bitrate);
 
 	return 0;
 }
@@ -768,41 +820,45 @@ static int a2dp_aac_sink_transport_start(struct ba_transport *t) {
 	return ba_transport_pcm_start(&t->a2dp.pcm, a2dp_aac_dec_thread, "ba-a2dp-aac");
 }
 
-struct a2dp_codec a2dp_aac_sink = {
-	.dir = A2DP_SINK,
-	.codec_id = A2DP_CODEC_MPEG24,
-	.synopsis = "A2DP Sink (AAC)",
-	.capabilities.aac = {
-		/* NOTE: AAC Long Term Prediction and AAC Scalable might be
-		 *       not supported by the FDK-AAC library. */
-		.object_type =
-			AAC_OBJECT_TYPE_MPEG2_LC |
-			AAC_OBJECT_TYPE_MPEG4_LC,
-		A2DP_AAC_INIT_FREQUENCY(
-				AAC_SAMPLING_FREQ_8000 |
-				AAC_SAMPLING_FREQ_11025 |
-				AAC_SAMPLING_FREQ_12000 |
-				AAC_SAMPLING_FREQ_16000 |
-				AAC_SAMPLING_FREQ_22050 |
-				AAC_SAMPLING_FREQ_24000 |
-				AAC_SAMPLING_FREQ_32000 |
-				AAC_SAMPLING_FREQ_44100 |
-				AAC_SAMPLING_FREQ_48000 |
-				AAC_SAMPLING_FREQ_64000 |
-				AAC_SAMPLING_FREQ_88200 |
-				AAC_SAMPLING_FREQ_96000)
-		.channels =
-			AAC_CHANNELS_1 |
-			AAC_CHANNELS_2,
-		.vbr = 1,
-		A2DP_AAC_INIT_BITRATE(320000)
+struct a2dp_sep a2dp_aac_sink = {
+	.name = "A2DP Sink (AAC)",
+	.config = {
+		.type = A2DP_SINK,
+		.codec_id = A2DP_CODEC_MPEG24,
+		.caps_size = sizeof(a2dp_aac_t),
+		.capabilities.aac = {
+			/* NOTE: AAC Long Term Prediction and AAC Scalable might be
+			 *       not supported by the FDK-AAC library. */
+			.object_type =
+				AAC_OBJECT_TYPE_MPEG2_LC |
+				AAC_OBJECT_TYPE_MPEG4_LC,
+			A2DP_AAC_INIT_SAMPLING_FREQ(
+					AAC_SAMPLING_FREQ_8000 |
+					AAC_SAMPLING_FREQ_11025 |
+					AAC_SAMPLING_FREQ_12000 |
+					AAC_SAMPLING_FREQ_16000 |
+					AAC_SAMPLING_FREQ_22050 |
+					AAC_SAMPLING_FREQ_24000 |
+					AAC_SAMPLING_FREQ_32000 |
+					AAC_SAMPLING_FREQ_44100 |
+					AAC_SAMPLING_FREQ_48000 |
+					AAC_SAMPLING_FREQ_64000 |
+					AAC_SAMPLING_FREQ_88200 |
+					AAC_SAMPLING_FREQ_96000)
+			/* NOTE: Other channel modes might be not supported
+			 *       by the FDK-AAC library. */
+			.channel_mode =
+				AAC_CHANNEL_MODE_MONO |
+				AAC_CHANNEL_MODE_STEREO,
+			.vbr = 1,
+			A2DP_AAC_INIT_BITRATE(320000)
+		},
 	},
-	.capabilities_size = sizeof(a2dp_aac_t),
 	.init = a2dp_aac_sink_init,
-	.capabilities_filter = a2dp_aac_capabilities_filter,
 	.configuration_select = a2dp_aac_configuration_select,
 	.configuration_check = a2dp_aac_configuration_check,
 	.transport_init = a2dp_aac_transport_init,
 	.transport_start = a2dp_aac_sink_transport_start,
+	.caps_helpers = &a2dp_aac_caps_helpers,
 	.enabled = true,
 };
