@@ -1,6 +1,6 @@
 /*
  * BlueALSA - ofono-iface.h
- * Copyright (c) 2016-2023 Arkadiusz Bokowy
+ * Copyright (c) 2016-2024 Arkadiusz Bokowy
  * Copyright (c) 2018 Thierry Bultel
  *
  * This file is a part of bluez-alsa.
@@ -13,6 +13,9 @@
 #define BLUEALSA_OFONOIFACE_H_
 
 #include <gio/gio.h>
+#include <glib.h>
+
+#include "dbus.h"
 
 #define OFONO_SERVICE "org.ofono"
 
@@ -25,13 +28,20 @@
 #define OFONO_AUDIO_CARD_TYPE_AG "gateway"
 #define OFONO_AUDIO_CARD_TYPE_HF "handsfree"
 
-#define OFONO_AUDIO_CODEC_CVSD 0x01
-#define OFONO_AUDIO_CODEC_MSBC 0x02
+#define OFONO_AUDIO_CODEC_CVSD    0x01
+#define OFONO_AUDIO_CODEC_MSBC    0x02
+#define OFONO_AUDIO_CODEC_LC3_SWB 0x03
 
 #define OFONO_MODEM_TYPE_HARDWARE "hardware"
 #define OFONO_MODEM_TYPE_HFP      "hfp"
 #define OFONO_MODEM_TYPE_SAP      "sap"
 
-extern const GDBusInterfaceInfo ofono_iface_hf_audio_agent;
+typedef struct {
+	GDBusInterfaceSkeletonEx parent;
+} OrgOfonoHandsfreeAudioAgentSkeleton;
+
+OrgOfonoHandsfreeAudioAgentSkeleton *org_ofono_handsfree_audio_agent_skeleton_new(
+		const GDBusInterfaceSkeletonVTable *vtable, void *userdata,
+		GDestroyNotify userdata_free_func);
 
 #endif
