@@ -1,6 +1,6 @@
 /*
  * BlueALSA - ofono.c
- * Copyright (c) 2016-2023 Arkadiusz Bokowy
+ * Copyright (c) 2016-2024 Arkadiusz Bokowy
  * Copyright (c) 2018 Thierry Bultel
  *
  * This file is a part of bluez-alsa.
@@ -17,9 +17,13 @@
  */
 
 #include "ofono.h"
-/* IWYU pragma: no_include "config.h" */
+
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include <errno.h>
+#include <poll.h>
 #include <pthread.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -32,7 +36,6 @@
 #include <unistd.h>
 
 #include <bluetooth/bluetooth.h>
-#include <bluetooth/hci.h>
 #include <bluetooth/hci_lib.h>
 
 #include <gio/gio.h>
@@ -41,15 +44,15 @@
 #include <glib.h>
 
 #include "ba-adapter.h"
+#include "ba-config.h"
 #include "ba-device.h"
 #include "ba-transport.h"
-#include "bluealsa-config.h"
+#include "ba-transport-pcm.h"
 #include "bluealsa-dbus.h"
 #include "dbus.h"
 #include "hci.h"
 #include "hfp.h"
 #include "ofono-iface.h"
-#include "ofono-skeleton.h"
 #include "utils.h"
 #include "shared/defs.h"
 #include "shared/log.h"
@@ -69,7 +72,7 @@ struct ofono_card_data {
 
 static GHashTable *ofono_card_data_map = NULL;
 static const char *dbus_agent_object_path = "/org/bluez/HFP/oFono";
-static ofono_HFAudioAgentIfaceSkeleton *dbus_hf_agent = NULL;
+static OrgOfonoHandsfreeAudioAgentSkeleton *dbus_hf_agent = NULL;
 
 /**
  * Ask oFono to connect to a card. */
@@ -79,7 +82,7 @@ static int ofono_acquire_bt_sco(struct ba_transport *t) {
 	GError *err = NULL;
 	uint8_t codec;
 	int fd = -1;
-	int ret = 0;
+	int ret = -1;
 
 	debug("Requesting new oFono SCO link: %s", t->sco.ofono_dbus_path_card);
 	msg = g_dbus_message_new_method_call(t->bluez_dbus_owner,
@@ -98,14 +101,10 @@ static int ofono_acquire_bt_sco(struct ba_transport *t) {
 	}
 
 	if ((rep = g_dbus_connection_send_message_with_reply_sync(config.dbus, msg,
-					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL)
+					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL ||
+			g_dbus_message_to_gerror(rep, &err))
 		goto fail;
 
-	if (g_dbus_message_get_message_type(rep) == G_DBUS_MESSAGE_TYPE_ERROR) {
-		g_dbus_message_to_gerror(rep, &err);
-		goto fail;
-	}
-
 	GVariant *body = g_dbus_message_get_body(rep);
 	g_variant_get(body, "(hy)", NULL, &codec);
 
@@ -129,17 +128,13 @@ static int ofono_acquire_bt_sco(struct ba_transport *t) {
 #endif
 
 	t->bt_fd = fd;
-	t->mtu_read = t->mtu_write = hci_sco_get_mtu(fd, t->d->a->hci.type);
+	t->mtu_read = t->mtu_write = hci_sco_get_mtu(fd, t->d->a);
 	ba_transport_set_codec(t, codec);
 
 	debug("New oFono SCO link (codec: %#x): %d", codec, fd);
-
-	goto final;
+	ret = 0;
 
 fail:
-	ret = -1;
-
-final:
 	if (msg != NULL)
 		g_object_unref(msg);
 	if (rep != NULL)
@@ -215,7 +210,8 @@ fail:
 
 /**
  * Lookup a transport associated with oFono card data. */
-static struct ba_transport *ofono_transport_lookup(struct ofono_card_data *ocd) {
+static struct ba_transport *ofono_transport_lookup(
+		const struct ofono_card_data *ocd) {
 
 	struct ba_adapter *a = NULL;
 	struct ba_device *d = NULL;
@@ -276,11 +272,11 @@ static void ofono_new_connection_finish(GObject *source, GAsyncResult *result,
 		void *userdata) {
 	(void)userdata;
 
+	GDBusMessage *rep;
 	GError *err = NULL;
-	GDBusMessage *rep = g_dbus_connection_send_message_with_reply_finish(
-			G_DBUS_CONNECTION(source), result, &err);
-	if (rep != NULL &&
-			g_dbus_message_get_message_type(rep) == G_DBUS_MESSAGE_TYPE_ERROR)
+
+	if ((rep = g_dbus_connection_send_message_with_reply_finish(
+					G_DBUS_CONNECTION(source), result, &err)) != NULL)
 		g_dbus_message_to_gerror(rep, &err);
 
 	if (rep != NULL)
@@ -328,14 +324,10 @@ static int ofono_card_link_modem(struct ofono_card_data *ocd) {
 			OFONO_IFACE_MANAGER, "GetModems");
 
 	if ((rep = g_dbus_connection_send_message_with_reply_sync(config.dbus, msg,
-					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL)
+					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL ||
+			g_dbus_message_to_gerror(rep, &err))
 		goto fail;
 
-	if (g_dbus_message_get_message_type(rep) == G_DBUS_MESSAGE_TYPE_ERROR) {
-		g_dbus_message_to_gerror(rep, &err);
-		goto fail;
-	}
-
 	GVariant *body = g_dbus_message_get_body(rep);
 
 	GVariantIter *modems;
@@ -421,8 +413,8 @@ fail:
 static unsigned int ofono_call_volume_property_sync(struct ba_transport *t,
 		const char *property, GVariant *value) {
 
-	struct ba_transport_pcm *spk = &t->sco.spk_pcm;
-	struct ba_transport_pcm *mic = &t->sco.mic_pcm;
+	struct ba_transport_pcm *spk = &t->sco.pcm_spk;
+	struct ba_transport_pcm *mic = &t->sco.pcm_mic;
 	unsigned int mask = OFONO_CALL_VOLUME_NONE;
 
 	if (strcmp(property, "Muted") == 0 &&
@@ -494,20 +486,16 @@ static int ofono_call_volume_get_properties(struct ba_transport *t) {
 
 	GDBusMessage *msg = NULL, *rep = NULL;
 	GError *err = NULL;
-	int ret = 0;
+	int ret = -1;
 
 	msg = g_dbus_message_new_method_call(t->bluez_dbus_owner,
 			t->sco.ofono_dbus_path_modem, OFONO_IFACE_CALL_VOLUME, "GetProperties");
 
 	if ((rep = g_dbus_connection_send_message_with_reply_sync(config.dbus, msg,
-					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL)
+					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL ||
+			g_dbus_message_to_gerror(rep, &err))
 		goto fail;
 
-	if (g_dbus_message_get_message_type(rep) == G_DBUS_MESSAGE_TYPE_ERROR) {
-		g_dbus_message_to_gerror(rep, &err);
-		goto fail;
-	}
-
 	GVariantIter *properties;
 	g_variant_get(g_dbus_message_get_body(rep), "(a{sv})", &properties);
 
@@ -521,17 +509,15 @@ static int ofono_call_volume_get_properties(struct ba_transport *t) {
 	}
 
 	if (mask & OFONO_CALL_VOLUME_SPEAKER)
-		bluealsa_dbus_pcm_update(&t->sco.spk_pcm, BA_DBUS_PCM_UPDATE_VOLUME);
+		bluealsa_dbus_pcm_update(&t->sco.pcm_spk, BA_DBUS_PCM_UPDATE_VOLUME);
 	if (mask & OFONO_CALL_VOLUME_MICROPHONE)
-		bluealsa_dbus_pcm_update(&t->sco.mic_pcm, BA_DBUS_PCM_UPDATE_VOLUME);
+		bluealsa_dbus_pcm_update(&t->sco.pcm_mic, BA_DBUS_PCM_UPDATE_VOLUME);
 
 	g_variant_iter_free(properties);
-	goto final;
+
+	ret = 0;
 
 fail:
-	ret = -1;
-
-final:
 	if (msg != NULL)
 		g_object_unref(msg);
 	if (rep != NULL)
@@ -550,7 +536,7 @@ static int ofono_call_volume_set_property(struct ba_transport *t,
 		const char *property, GVariant *value, GError **error) {
 
 	GDBusMessage *msg = NULL, *rep = NULL;
-	int ret = 0;
+	int ret = -1;
 
 	msg = g_dbus_message_new_method_call(t->bluez_dbus_owner,
 			t->sco.ofono_dbus_path_modem, OFONO_IFACE_CALL_VOLUME, "SetProperty");
@@ -558,20 +544,13 @@ static int ofono_call_volume_set_property(struct ba_transport *t,
 	g_dbus_message_set_body(msg, g_variant_new("(sv)", property, value));
 
 	if ((rep = g_dbus_connection_send_message_with_reply_sync(config.dbus, msg,
-					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, error)) == NULL)
+					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, error)) == NULL ||
+			g_dbus_message_to_gerror(rep, error))
 		goto fail;
 
-	if (g_dbus_message_get_message_type(rep) == G_DBUS_MESSAGE_TYPE_ERROR) {
-		g_dbus_message_to_gerror(rep, error);
-		goto fail;
-	}
-
-	goto final;
+	ret = 0;
 
 fail:
-	ret = -1;
-
-final:
 	if (msg != NULL)
 		g_object_unref(msg);
 	if (rep != NULL)
@@ -684,20 +663,16 @@ static int ofono_get_all_cards(void) {
 
 	GDBusMessage *msg = NULL, *rep = NULL;
 	GError *err = NULL;
-	int ret = 0;
+	int ret = -1;
 
 	msg = g_dbus_message_new_method_call(OFONO_SERVICE, "/",
 			OFONO_IFACE_HF_AUDIO_MANAGER, "GetCards");
 
 	if ((rep = g_dbus_connection_send_message_with_reply_sync(config.dbus, msg,
-					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL)
+					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL ||
+			g_dbus_message_to_gerror(rep, &err))
 		goto fail;
 
-	if (g_dbus_message_get_message_type(rep) == G_DBUS_MESSAGE_TYPE_ERROR) {
-		g_dbus_message_to_gerror(rep, &err);
-		goto fail;
-	}
-
 	const char *sender = g_dbus_message_get_sender(rep);
 	GVariant *body = g_dbus_message_get_body(rep);
 
@@ -712,12 +687,10 @@ static int ofono_get_all_cards(void) {
 	}
 
 	g_variant_iter_free(cards);
-	goto final;
+
+	ret = 0;
 
 fail:
-	ret = -1;
-
-final:
 	if (msg != NULL)
 		g_object_unref(msg);
 	if (rep != NULL)
@@ -800,18 +773,24 @@ static void ofono_agent_new_connection(GDBusMethodInvocation *inv, void *userdat
 		g_dbus_method_invocation_return_value(inv, NULL);
 		return;
 	}
+#endif
 
 	/* For HF, oFono does not authorize after setting the voice option, so we
-	 * have to do it ourselves here. */
-	if (t->profile == BA_TRANSPORT_PROFILE_HFP_HF &&
-			codec == HFP_CODEC_MSBC) {
+	 * may have to do it ourselves here. oFono always tries to set the
+	 * BT_DEFER_SETUP option, but may not always succeed. So we must first
+	 * check if the socket is in deferred setup state before authorizing. */
+	if (t->profile == BA_TRANSPORT_PROFILE_HFP_HF) {
+		/* If socket is not writable, it means that it is in the defer setup
+		 * state, so it needs to be read to authorize the connection. */
+		struct pollfd pfd = { fd, POLLOUT, 0 };
 		uint8_t auth;
-		if (read(fd, &auth, sizeof(auth)) == -1) {
+		if (poll(&pfd, 1, 0) == -1)
+			goto fail;
+		if (!(pfd.revents & POLLOUT) && read(fd, &auth, sizeof(auth)) == -1) {
 			error("Couldn't authorize oFono SCO link: %s", strerror(errno));
 			goto fail;
 		}
 	}
-#endif
 
 	ba_transport_stop(t);
 
@@ -820,13 +799,13 @@ static void ofono_agent_new_connection(GDBusMethodInvocation *inv, void *userdat
 	debug("New oFono SCO link (codec: %#x): %d", codec, fd);
 
 	t->bt_fd = fd;
-	t->mtu_read = t->mtu_write = hci_sco_get_mtu(fd, t->d->a->hci.type);
+	t->mtu_read = t->mtu_write = hci_sco_get_mtu(fd, t->d->a);
 	ba_transport_set_codec(t, codec);
 
 	pthread_mutex_unlock(&t->bt_fd_mtx);
 
-	ba_transport_thread_state_set_idle(&t->thread_enc);
-	ba_transport_thread_state_set_idle(&t->thread_dec);
+	ba_transport_pcm_state_set_idle(&t->sco.pcm_spk);
+	ba_transport_pcm_state_set_idle(&t->sco.pcm_mic);
 	ba_transport_start(t);
 
 	g_dbus_method_invocation_return_value(inv, NULL);
@@ -873,18 +852,18 @@ int ofono_register(void) {
 		.dispatchers = dispatchers,
 	};
 
+	if (!config.profile.hfp_ofono)
+		return 0;
+
 	GDBusMessage *msg = NULL, *rep = NULL;
 	GError *err = NULL;
-	int ret = 0;
-
-	if (!config.profile.hfp_ofono)
-		goto final;
+	int ret = -1;
 
 	debug("Registering oFono audio agent: %s", dbus_agent_object_path);
 
 	if (dbus_hf_agent == NULL) {
-		ofono_HFAudioAgentIfaceSkeleton *ifs_hf_agent;
-		if ((ifs_hf_agent = ofono_hf_audio_agent_iface_skeleton_new(&vtable, NULL, NULL)) == NULL)
+		OrgOfonoHandsfreeAudioAgentSkeleton *ifs_hf_agent;
+		if ((ifs_hf_agent = org_ofono_handsfree_audio_agent_skeleton_new(&vtable, NULL, NULL)) == NULL)
 			goto fail;
 		GDBusInterfaceSkeleton *ifs = G_DBUS_INTERFACE_SKELETON(ifs_hf_agent);
 		if (!g_dbus_interface_skeleton_export(ifs, config.dbus, dbus_agent_object_path, &err)) {
@@ -911,22 +890,15 @@ int ofono_register(void) {
 	g_variant_builder_clear(&options);
 
 	if ((rep = g_dbus_connection_send_message_with_reply_sync(config.dbus, msg,
-					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL)
+					G_DBUS_SEND_MESSAGE_FLAGS_NONE, -1, NULL, NULL, &err)) == NULL ||
+			g_dbus_message_to_gerror(rep, &err))
 		goto fail;
 
-	if (g_dbus_message_get_message_type(rep) == G_DBUS_MESSAGE_TYPE_ERROR) {
-		g_dbus_message_to_gerror(rep, &err);
-		goto fail;
-	}
-
 	ofono_get_all_cards();
 
-	goto final;
+	ret = 0;
 
 fail:
-	ret = -1;
-
-final:
 	if (msg != NULL)
 		g_object_unref(msg);
 	if (rep != NULL)
@@ -1011,9 +983,9 @@ static void ofono_signal_volume_changed(GDBusConnection *conn, const char *sende
 
 	unsigned int mask = ofono_call_volume_property_sync(t, property, value);
 	if (mask & OFONO_CALL_VOLUME_SPEAKER)
-		bluealsa_dbus_pcm_update(&t->sco.spk_pcm, BA_DBUS_PCM_UPDATE_VOLUME);
+		bluealsa_dbus_pcm_update(&t->sco.pcm_spk, BA_DBUS_PCM_UPDATE_VOLUME);
 	if (mask & OFONO_CALL_VOLUME_MICROPHONE)
-		bluealsa_dbus_pcm_update(&t->sco.mic_pcm, BA_DBUS_PCM_UPDATE_VOLUME);
+		bluealsa_dbus_pcm_update(&t->sco.pcm_mic, BA_DBUS_PCM_UPDATE_VOLUME);
 
 	g_variant_unref(value);
 	ba_transport_unref(t);
@@ -1103,11 +1075,11 @@ bool ofono_detect_service(void) {
  * Update oFono call volume properties. */
 int ofono_call_volume_update(struct ba_transport *t) {
 
-	struct ba_transport_pcm *spk = &t->sco.spk_pcm;
-	struct ba_transport_pcm *mic = &t->sco.mic_pcm;
+	struct ba_transport_pcm *spk = &t->sco.pcm_spk;
+	struct ba_transport_pcm *mic = &t->sco.pcm_mic;
 	int ret = 0;
 
-	struct {
+	const struct {
 		const char *name;
 		GVariant *value;
 	} props[] = {
